#include "branch_selector.h"
#include "config_manager.h"

wxBEGIN_EVENT_TABLE(BranchSelector, wxPanel)
    EVT_COMBOBOX(wxID_ANY, BranchSelector::OnBranchSelected)
wxEND_EVENT_TABLE()

BranchSelector::BranchSelector(wxWindow* parent, std::shared_ptr<git::Repository> repo,
                             const wxString& label)
    : wxPanel(parent, wxID_ANY), repo_(repo) {

    wxBoxSizer* hbox = new wxBoxSizer(wxHORIZONTAL);

    // 如果没有提供标签，使用存储库名称
    wxString display_label = label.IsEmpty() ? wxString(repo->GetName()) : label;
    wxStaticText* label_ctrl = new wxStaticText(this, wxID_ANY, display_label);

    // 创建下拉列表
    branch_combo_ = new wxComboBox(this, wxID_ANY);

    // 填充分支列表
    const auto& branches = repo->GetAllBranches();
    for (const auto& branch : branches) {
        branch_combo_->Append(branch);
    }

    // 选择当前分支
    branch_combo_->SetStringSelection(repo->GetCurrentBranchName());

    // 布局
    hbox->Add(label_ctrl, 0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5);
    hbox->Add(branch_combo_, 1);

    SetSizerAndFit(hbox);
}

bool BranchSelector::SelectBranch(const wxString& branch_name) {
    return branch_combo_->SetStringSelection(branch_name);
}

bool BranchSelector::SelectGroupBranch(const wxString& group_name) {
    current_group_ = group_name;

    // 尝试从配置中获取分支
    bool found = false;
    std::string branch_name = ConfigManager::Instance().GetBranchFromGroup(
        group_name.ToStdString(),
        repo_->GetName(),
        "Develop",
        &found
    );

    // 如果找到了缓存，使用缓存，否则使用组名
    if (found) {
        if (!branch_combo_->SetStringSelection(branch_name)) {
            // 如果设置失败，可能是因为分支不存在，尝试使用组名
            return branch_combo_->SetStringSelection(group_name);
        }
        return true;
    } else {
        // 尝试使用组名作为分支名
        if (!branch_combo_->SetStringSelection(group_name)) {
            // 如果组名设置失败，使用默认分支
            return branch_combo_->SetStringSelection(branch_name);
        }
        return true;
    }
}

std::string BranchSelector::GetSelectedBranchName() const {
    return branch_combo_->GetStringSelection().ToStdString();
}

bool BranchSelector::ApplyBranchChange(bool hard_reset) {
    std::string selected_branch = GetSelectedBranchName();
    if (selected_branch.empty()) {
        return false;
    }

    // 切换分支
    bool success = repo_->SwitchBranch(selected_branch, hard_reset);
    if (!success) {
        return false;
    }

    // 拉取更新
    return repo_->Pull();
}

void BranchSelector::OnBranchSelected(wxCommandEvent& event) {
    // 如果选择的分支不是组名，保存到配置
    if (!current_group_.IsEmpty() && event.GetString() != current_group_) {
        ConfigManager::Instance().SaveBranchToGroup(
            current_group_.ToStdString(),
            repo_->GetName(),
            event.GetString().ToStdString()
        );
    }
}
