#include "git_repository.h"
#include <iostream>
#include <sstream>

/**
 * @brief 检查 libgit2 函数调用的错误代码。
 *
 * 如果错误代码指示失败（通常小于 0），则获取最后的 libgit2 错误信息，
 * 构造一个包含操作描述和错误详情的字符串，并抛出 std::runtime_error 异常。
 *
 * @param error_code libgit2 函数返回的整数错误代码。
 * @param action_description 描述执行失败的操作的字符串（例如，“打开仓库”、“拉取更新”）。
 * @throws std::runtime_error 如果 error_code 表示失败。
 */
void CheckGitError(int error_code, const std::string& action_description) {
    // libgit2 通常在出错时返回负值
    if (error_code < 0) {
        // 获取 libgit2 线程本地存储中的最后一个错误信息
        const git_error* last_error = git_error_last();

        // 构建详细的错误消息
        std::string error_message = action_description + " 失败: ";

        // 检查 last_error 是否有效以及是否有错误消息文本
        if (last_error && last_error->message) {
            error_message += last_error->message;
        } else {
            // 如果 libgit2 没有提供具体的错误信息
            error_message += "未知错误 (libgit2 未提供详细信息)";
        }

        // 可以选择性地附加错误代码以供调试
        error_message += " (错误码: " + std::to_string(error_code) + ")";

        // 抛出标准运行时异常
        throw std::runtime_error(error_message);
    }
    // 如果 error_code >= 0，则表示操作成功，函数不执行任何操作
}

namespace git {
    // --- 假设 RAII 封装类已存在 ---
    // GitRemote 示例:
    class GitRemote : public GitResource {
    public:
        explicit GitRemote(git_remote* remote) : remote_(remote) {}
        ~GitRemote() override { if (remote_) git_remote_free(remote_); }
        operator git_remote*() const { return remote_; }
    private:
        git_remote* remote_ = nullptr;
    };

    // GitAnnotatedCommit 示例:
    class GitAnnotatedCommit : public GitResource {
    public:
        explicit GitAnnotatedCommit(git_annotated_commit* commit) : commit_(commit) {}
        ~GitAnnotatedCommit() override { if (commit_) git_annotated_commit_free(commit_); }
        operator git_annotated_commit*() const { return commit_; }
        const git_oid* Id() const { return commit_ ? git_annotated_commit_id(commit_) : nullptr; }
    private:
        git_annotated_commit* commit_ = nullptr;
    };

    // GitIndex 示例
    class GitIndex : public GitResource {
    public:
        explicit GitIndex(git_index *index) : index_(index) {}
        ~GitIndex() override { if (index_) git_index_free(index_); }
        operator git_index*() const { return index_; }
        bool HasConflicts() const { return index_ && git_index_has_conflicts(index_); }
        int Write() { return index_ ? git_index_write(index_) : -1; }
        int ReadTree(const git_tree* tree) { return index_ ? git_index_read_tree(index_, tree) : -1; }
        int WriteTree(git_oid* out_oid) { return index_ ? git_index_write_tree(out_oid, index_) : -1; }
    private:
        git_index* index_ = nullptr;
    };

    // GitCommit 示例
    class GitCommit : public GitResource {
    public:
        explicit GitCommit(git_commit *commit) : commit_(commit) {}
        ~GitCommit() override { if (commit_) git_commit_free(commit_); }
        operator git_commit*() const { return commit_; }
        const git_oid* Id() const { return commit_ ? git_commit_id(commit_) : nullptr; }
        const git_signature* Committer() const { return commit_ ? git_commit_committer(commit_) : nullptr; }
        const git_signature* Author() const { return commit_ ? git_commit_author(commit_) : nullptr; }
        const char* Message() const { return commit_ ? git_commit_message(commit_) : ""; }
        git_tree* Tree() const {
            git_tree* tree = nullptr;
            if (commit_ && git_commit_tree(&tree, commit_) == 0) {
                return tree; // 调用者必须管理此 tree 的生命周期或将其包装
            }
            return nullptr;
        }
    private:
        git_commit* commit_ = nullptr;
    };

    // GitTree 示例
    class GitTree : public GitResource {
    public:
        explicit GitTree(git_tree *tree) : tree_(tree) {}
        ~GitTree() override { if (tree_) git_tree_free(tree_); }
        operator git_tree*() const { return tree_; }
        const git_oid* Id() const { return tree_ ? git_tree_id(tree_) : nullptr; }
    private:
        git_tree* tree_ = nullptr;
    };

    // GitSignature 示例
    class GitSignature : public GitResource {
    public:
        explicit GitSignature(git_signature *sig) : sig_(sig) {}
        ~GitSignature() override { if (sig_) git_signature_free(sig_); }
        operator git_signature*() const { return sig_; }
    private:
        git_signature* sig_ = nullptr;
    };
    // --- RAII 封装类结束 ---

    // GitReference 实现
    std::string GitReference::GetShortName() const {
        if (!ref_) return "";
        return git_reference_shorthand(ref_);
    }

    // Repository 实现
    Repository::Repository(git_repository* repo) : repo_(repo) {
        if (repo_) {
            path_ = git_repository_path(repo_);

            // 默认仓库名为目录名
            size_t last_slash = path_.find_last_of("/\\");
            if (last_slash != std::string::npos) {
                name_ = path_.substr(last_slash + 1);
                // 如果路径以 .git 结尾，去掉它
                size_t git_suffix = name_.find(".git");
                if (git_suffix != std::string::npos && git_suffix == name_.size() - 4) {
                    name_ = name_.substr(0, git_suffix);
                }
            } else {
                name_ = path_;
            }
        }
    }

    Repository::~Repository() {
        if (repo_) {
            git_repository_free(repo_);
            repo_ = nullptr;
        }
    }

    std::string Repository::GetCurrentBranchName() const {
        if (!repo_) return "";

        git_reference* head = nullptr;
        int error = git_repository_head(&head, repo_);
        if (error != 0) {
            const git_error* e = git_error_last();
            std::cerr << "Error getting HEAD: " << (e ? e->message : "unknown error") << std::endl;
            return "";
        }

        GitReference ref(head);
        return ref.GetShortName();
    }

    void ForEachBranch(git_repository* repo, git_branch_t flags,
                      const std::function<void(const GitReference&)>& callback) {
        git_branch_iterator* iter = nullptr;
        if (git_branch_iterator_new(&iter, repo, flags) != 0) {
            return;
        }

        git_reference* ref = nullptr;
        git_branch_t type;
        while (git_branch_next(&ref, &type, iter) == 0) {
            GitReference git_ref(ref);
            callback(git_ref);
        }

        git_branch_iterator_free(iter);
    }

    std::vector<std::string> Repository::GetAllBranches() const {
        std::vector<std::string> branches;
        if (!repo_) return branches;

        ForEachBranch(repo_, GIT_BRANCH_ALL, [&branches](const GitReference& ref) {
            const char* name;
            if (git_branch_name(&name, ref) == 0) {
                branches.push_back(name);
            }
        });

        return branches;
    }

    bool Repository::SwitchBranch(const std::string& branch_name, bool hard_reset) {
        if (!repo_) return false;

        try {
            // 尝试查找本地分支
            git_reference* target_ref = nullptr;
            int error = git_branch_lookup(&target_ref, repo_, branch_name.c_str(), GIT_BRANCH_LOCAL);
            if (error != 0) {
                // 尝试查找远程分支
                error = git_branch_lookup(&target_ref, repo_, branch_name.c_str(), GIT_BRANCH_REMOTE);
                if (error != 0) {
                    std::cerr << "Branch not found: " << branch_name << std::endl;
                    return false;
                }
            }

            GitReference ref(target_ref);

            // 获取目标commit
            git_object* target_commit = nullptr;
            error = git_reference_peel(&target_commit, ref, GIT_OBJECT_COMMIT);
            if (error != 0) {
                std::cerr << "Failed to peel reference to commit" << std::endl;
                return false;
            }

            // 创建检出选项
            git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
            checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE;

            // 检出分支
            error = git_checkout_tree(repo_, target_commit, &checkout_opts);
            if (error != 0) {
                git_object_free(target_commit);
                std::cerr << "Failed to checkout tree" << std::endl;
                return false;
            }

            git_object_free(target_commit);

            // 更新HEAD引用
            error = git_repository_set_head(repo_, git_reference_name(ref));
            if (error != 0) {
                std::cerr << "Failed to update HEAD reference" << std::endl;
                return false;
            }

            // 如果需要硬重置
            if (hard_reset) {
                git_reference* remote_ref = nullptr;
                std::string remote_branch_name = "origin/" + branch_name;
                error = git_branch_lookup(&remote_ref, repo_, remote_branch_name.c_str(), GIT_BRANCH_REMOTE);
                if (error == 0) {
                    GitReference remote(remote_ref);

                    git_object* remote_commit = nullptr;
                    error = git_reference_peel(&remote_commit, remote, GIT_OBJECT_COMMIT);
                    if (error == 0) {
                        // 重置到远程分支
                        error = git_reset(repo_, remote_commit, GIT_RESET_HARD, nullptr);
                        git_object_free(remote_commit);

                        if (error != 0) {
                            std::cerr << "Failed to hard reset" << std::endl;
                            return false;
                        }
                    }
                }
            }

            return true;
        }
        catch (const std::exception& e) {
            std::cerr << "Exception during branch switch: " << e.what() << std::endl;
            return false;
        }
    }

    bool Repository::Pull() {
        if (!repo_) {
            std::cerr << "Pull 错误：仓库未打开。" << std::endl;
            return false;
        }

        // 初始化原始指针，稍后由 RAII 接管
        git_reference* head_ref_ptr = nullptr;
        git_annotated_commit* remote_head_annotated_ptr = nullptr;
        git_remote* remote_ptr = nullptr;

        try {
            // 1. 获取当前的 HEAD 引用
            CheckGitError(git_repository_head(&head_ref_ptr, repo_), "获取 HEAD 引用");
            GitReference head_ref(head_ref_ptr); // RAII 封装

            if (!git_reference_is_branch(head_ref)) {
                std::cerr << "Pull 警告：HEAD 处于分离状态。无法执行 pull。" << std::endl;
                // 这不完全是错误，但 pull 在此无意义
                return true; // 或根据期望行为返回 false
            }
            std::cout << "当前分支: " << git_reference_shorthand(head_ref) << std::endl;

            // 2. 获取当前 HEAD 的上游分支引用
            git_reference* upstream_ref_ptr = nullptr;
            int upstream_error = git_branch_upstream(&upstream_ref_ptr, head_ref);
            if (upstream_error == GIT_ENOTFOUND) {
                std::cerr << "Pull 错误：分支 '" << git_reference_shorthand(head_ref)
                          << "' 未配置上游分支。" << std::endl;
                return false;
            }
            CheckGitError(upstream_error, "获取上游分支");
            GitReference upstream_ref(upstream_ref_ptr); // RAII 封装

            const char* upstream_name = git_reference_name(upstream_ref); // e.g., "refs/remotes/origin/main"
            if (!upstream_name) {
                throw std::runtime_error("无法获取上游引用名称。");
            }
            std::cout << "上游分支引用: " << upstream_name << std::endl;


            // 3. 从上游引用中获取远程仓库名称和远程分支名称
            char    remote_name_buffer[256]; // 假设远程名称不会太长
            git_buf remote_name_buf = { remote_name_buffer, 0, sizeof(remote_name_buffer), };
            CheckGitError(git_branch_remote_name(&remote_name_buf, repo_, upstream_name), "获取远程仓库名称");
            std::string remote_name_str(remote_name_buffer); // 例如 "origin"

            if (remote_name_str.empty()) {
                throw std::runtime_error("无法从上游引用确定远程仓库名称。");
            }
            std::cout << "远程仓库名称: " << remote_name_str << std::endl;

            // 提取远程分支名称 (例如从 "refs/remotes/origin/main" 提取 "main")
            std::string remote_branch_name_str;
            const char* shorthand = git_reference_shorthand(upstream_ref); // 例如 "origin/main"
            if (shorthand) {
                std::string shorthand_str(shorthand);
                size_t slash_pos = shorthand_str.find('/');
                if (slash_pos != std::string::npos) {
                    remote_branch_name_str = shorthand_str.substr(slash_pos + 1);
                }
            }
            if (remote_branch_name_str.empty()) {
                throw std::runtime_error("无法从上游引用确定远程分支名称。");
            }
            std::cout << "远程分支名称: " << remote_branch_name_str << std::endl;


            // 4. 查找远程仓库
            CheckGitError(git_remote_lookup(&remote_ptr, repo_, remote_name_str.c_str()), "查找远程仓库 '" + remote_name_str + "'");
            GitRemote remote(remote_ptr); // RAII 封装

            // 5. 从远程仓库 Fetch
            std::cout << "正在从 " << remote_name_str << " 拉取..." << std::endl;
            git_fetch_options fetch_opts = GIT_FETCH_OPTIONS_INIT;
            // 您可能需要配置 fetch_opts，例如设置凭据回调
            // fetch_opts.callbacks.credentials = ...;
            CheckGitError(git_remote_fetch(remote, nullptr, &fetch_opts, "fetch: Pull 过程中自动 fetch"), "从远程仓库 Fetch");
            std::cout << "Fetch 完成。" << std::endl;

            // 6. 获取 Fetch 后的远程 HEAD 的 Annotated Commit
            // upstream_ref 现在可能指向 *新* Fetch 下来的 commit ID
            CheckGitError(git_annotated_commit_from_ref(&remote_head_annotated_ptr, repo_, upstream_ref), "获取远程 HEAD 的 Annotated Commit");
            GitAnnotatedCommit remote_head_annotated(remote_head_annotated_ptr); // RAII 封装

            // 7. 执行合并分析 (Merge Analysis)
            git_merge_analysis_t analysis_out;
            git_merge_preference_t preference_out;
            const git_annotated_commit* merge_heads[] = {remote_head_annotated};

            CheckGitError(git_merge_analysis(&analysis_out, &preference_out, repo_, merge_heads, 1), "执行合并分析");

            // 8. 处理不同的合并场景
            if (analysis_out & GIT_MERGE_ANALYSIS_UP_TO_DATE) {
                std::cout << "分支 '" << git_reference_shorthand(head_ref)
                          << "' 已经是最新的。" << std::endl;
                return true;
            } else {
                if (analysis_out & GIT_MERGE_ANALYSIS_FASTFORWARD) {
                    std::cout << "执行快进 (Fast-forward) 合并..." << std::endl;

                    // 从 annotated commit 获取目标 commit OID
                    const git_oid* target_oid = remote_head_annotated.Id();
                    if (!target_oid) throw std::runtime_error("无法从远程 annotated commit 获取 OID。");

                    // 获取本地分支 (HEAD) 的引用对象
                    git_reference* local_branch_ref_ptr = nullptr;
                    CheckGitError(git_reference_lookup(&local_branch_ref_ptr, repo_, git_reference_name(head_ref)), "查找本地分支引用以进行快进合并");
                    GitReference local_branch_ref(local_branch_ref_ptr); // RAII

                    // 更新本地分支引用，使其直接指向远程 commit
                    git_reference* new_direct_ref_ptr = nullptr;
                    CheckGitError(git_reference_set_target(&new_direct_ref_ptr, local_branch_ref, target_oid, "pull: 快进更新。"), "快进合并期间设置本地分支目标");
                    // 如果需要管理 new_direct_ref_ptr 的生命周期，也用 RAII 包装
                    if (new_direct_ref_ptr) git_reference_free(new_direct_ref_ptr);


                    // Checkout HEAD 以更新工作目录和索引
                    git_checkout_options ff_checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
                    ff_checkout_opts.checkout_strategy    = GIT_CHECKOUT_FORCE; // 使用 FORCE 确保工作目录匹配
                    CheckGitError(git_checkout_head(repo_, &ff_checkout_opts), "快进合并后检出 HEAD");

                    std::cout << "快进合并成功完成。" << std::endl;
                    return true;

                }
                if (analysis_out & GIT_MERGE_ANALYSIS_NORMAL) {
                    std::cout << "执行普通合并..." << std::endl;

                    git_merge_options    merge_opts    = GIT_MERGE_OPTIONS_INIT;
                    git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
                    // 倾向于创建包含冲突的文件，而不是立即中止
                    checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE | GIT_CHECKOUT_RECREATE_MISSING | GIT_CHECKOUT_ALLOW_CONFLICTS;

                    CheckGitError(git_merge(repo_, merge_heads, 1, &merge_opts, &checkout_opts),
                                  "执行合并操作");

                    // 检查是否存在冲突
                    git_index* index_ptr = nullptr;
                    CheckGitError(git_repository_index(&index_ptr, repo_), "合并后获取仓库索引");
                    GitIndex index(index_ptr); // RAII 封装

                    if (index.HasConflicts()) {
                        std::cerr << "Pull 错误：检测到合并冲突。请手动解决冲突并提交。" << std::endl;
                        // 将仓库留在合并状态供用户解决
                        // 如果需要，可以在此处保存冲突细节
                        CheckGitError(index.Write(), "写入包含冲突的索引"); // 保存包含冲突标记的索引状态
                        return false;                              // 因冲突而指示失败
                    } else {
                        std::cout << "合并完成，无冲突。正在创建合并提交..." << std::endl;

                        // 创建合并提交 (Merge Commit)
                        git_oid tree_oid;
                        CheckGitError(index.WriteTree(&tree_oid), "写入合并后的树"); // 将索引更改写入树对象

                        git_tree* tree_ptr = nullptr;
                        CheckGitError(git_tree_lookup(&tree_ptr, repo_, &tree_oid), "查找合并后的树");
                        GitTree merged_tree(tree_ptr); // RAII 封装

                        // 创建签名 (Signature)
                        git_signature* signature_ptr = nullptr;
                        // !! 重要：替换为实际的用户名和邮箱，最好从 Git 配置读取
                        CheckGitError(git_signature_default(&signature_ptr, repo_), "获取默认签名");
                        // 如果 git_signature_default 失败或未配置，需要手动创建或提供默认值
                        if(!signature_ptr) {
                            CheckGitError(git_signature_now(&signature_ptr, "默认用户名", "user@example.com"), "创建签名");
                        }
                        GitSignature signature(signature_ptr); // RAII 封装

                        // 获取本地 HEAD commit 作为父提交
                        git_commit* local_head_commit_ptr = nullptr;
                        git_oid     local_head_oid;
                        CheckGitError(git_reference_name_to_id(&local_head_oid, repo_, git_reference_name(head_ref)),
                                      "获取本地 HEAD OID 用于合并提交");
                        CheckGitError(git_commit_lookup(&local_head_commit_ptr, repo_, &local_head_oid),
                                      "查找本地 HEAD commit 用于合并提交");
                        GitCommit local_head_commit(local_head_commit_ptr); // RAII

                        // 获取远程 commit 作为另一个父提交
                        git_commit* remote_commit_ptr = nullptr;
                        CheckGitError(git_commit_lookup(&remote_commit_ptr, repo_, remote_head_annotated.Id()),
                                      "查找远程 commit 用于合并提交");
                        GitCommit remote_commit(remote_commit_ptr); // RAII


                        git_commit* parents[] = {local_head_commit, remote_commit};
                        // 生成合并提交消息
                        std::string commit_message = "Merge remote-tracking branch '" + remote_name_str + "/" + remote_branch_name_str + "'";

                        // 创建合并提交
                        git_oid new_commit_oid;
                        CheckGitError(git_commit_create(&new_commit_oid, repo_,
                                                        git_reference_name(head_ref), // 更新哪个引用 (HEAD)
                                                        signature, signature,         // 作者和提交者签名
                                                        nullptr,                      // 编码，nullptr 表示 UTF-8
                                                        commit_message.c_str(),       // 提交信息
                                                        merged_tree,                  // 合并后的树
                                                        2, parents),                  // 父提交数量和数组
                                      "创建合并提交");

                        // 清理仓库状态（例如，移除 MERGE_HEAD 文件）
                        CheckGitError(git_repository_state_cleanup(repo_), "合并后清理仓库状态");

                        std::cout << "合并提交成功创建。" << std::endl;
                        return true;
                    }
                } else if (analysis_out & GIT_MERGE_ANALYSIS_UNBORN) {
                    std::cerr << "Pull 错误：本地分支 '" << git_reference_shorthand(head_ref)
                            << "' 尚未出生 (unborn)。无法合并。" << std::endl;
                    return false;
                } else {
                    // 其他未知的分析结果
                    std::cerr << "Pull 错误：未知的合并分析结果 (" << analysis_out << ")。" << std::endl;
                    return false;
                }
            }
        } catch (const std::exception& e) {
            std::cerr << "Pull 操作失败: " << e.what() << std::endl;
            // 清理在部分失败情况下可能未被 RAII 覆盖的资源
            // （尽管如果 RAII 实现正确，大多数情况应已处理）
            return false;
        }
        // 当 RAII 包装器离开作用域时，它们会自动释放资源（如 head_ref, upstream_ref, remote 等）
        // 即使发生异常也是如此。
    }

    int OpenSubmodule(git_submodule* submodule, const char* name, void* payload) {
        auto submodules = static_cast<std::vector<std::shared_ptr<Repository>>*>(payload);

        git_repository* subrepo = nullptr;
        if (git_submodule_open(&subrepo, submodule) == 0) {
            auto repo = std::make_shared<Repository>(subrepo);
            repo->SetName(name);
            submodules->push_back(repo);
        }

        return 0;
    }

    std::vector<std::shared_ptr<Repository>> Repository::GetSubmodules() const {
        std::vector<std::shared_ptr<Repository>> submodules;
        if (!repo_) return submodules;

        git_submodule_foreach(repo_, OpenSubmodule, &submodules);
        return submodules;
    }

    bool InitLibGit() {
        return git_libgit2_init() >= 0;
    }

    void CleanupLibGit() {
        git_libgit2_shutdown();
    }

    bool IsValidRepository(const std::string& path) {
        git_repository* repo = nullptr;
        int error = git_repository_open(&repo, path.c_str());
        if (error == 0) {
            git_repository_free(repo);
            return true;
        }
        return false;
    }

    std::shared_ptr<Repository> OpenRepository(const std::string& path) {
        git_repository* repo = nullptr;
        int error = git_repository_open(&repo, path.c_str());
        if (error != 0) {
            return nullptr;
        }

        return std::make_shared<Repository>(repo);
    }
} // namespace git
