#pragma once
#include <git2.h>
#include <string>
#include <vector>
#include <memory>
#include <functional>

namespace git {

	// 为libgit2对象提供RAII包装的类
	class GitResource {
	public:
		virtual ~GitResource() = default;
	};

	// Git引用的RAII包装
	class GitReference : public GitResource {
	public:
		explicit GitReference(git_reference* ref) : ref_(ref) {}
		~GitReference() override { if (ref_) git_reference_free(ref_); }

		operator git_reference*() const { return ref_; }
		std::string GetShortName() const;

	private:
		git_reference* ref_ = nullptr;
	};

	// Git分支操作类
	class Branch {
	public:
		explicit Branch(const std::string& name) : name_(name) {}

		const std::string& GetName() const { return name_; }

	private:
		std::string name_;
	};

	// Git仓库操作类
	class Repository {
	public:
		explicit Repository(git_repository* repo);
		~Repository();

		Repository(const Repository&) = delete;
		Repository& operator=(const Repository&) = delete;

		// 基本属性
		const std::string& GetName() const { return name_; }
		const std::string& GetPath() const { return path_; }
		void SetName(const std::string& name) { name_ = name; }

		// 分支操作
		std::string GetCurrentBranchName() const;
		std::vector<std::string> GetAllBranches() const;
		bool SwitchBranch(const std::string& branch_name, bool hard_reset = false);
		bool Pull();

		// 子模块操作
		std::vector<std::shared_ptr<Repository>> GetSubmodules() const;

		// 转换操作符
		operator git_repository*() const { return repo_; }
		operator bool() const { return repo_ != nullptr; }

	private:
		git_repository* repo_ = nullptr;
		std::string name_;
		std::string path_;
	};

	// 初始化和清理
	bool InitLibGit();
	void CleanupLibGit();

	// 辅助函数
	bool IsValidRepository(const std::string& path);
	std::shared_ptr<Repository> OpenRepository(const std::string& path);

} // namespace git
