#include "main_frame.h"
#include "config_manager.h"

wxBEGIN_EVENT_TABLE(MainFrame, wxFrame)
    EVT_BUTTON(wxID_APPLY, MainFrame::OnApply)
    EVT_COMBOBOX(wxID_ANY, MainFrame::OnGroupSelect)
    EVT_CHECKBOX(wxID_ANY, MainFrame::OnHardResetChanged)
wxEND_EVENT_TABLE()

MainFrame::MainFrame(const wxString& title)
    : wxFrame(nullptr, wxID_ANY, title, wxDefaultPosition, wxDefaultSize) {

    // 加载主仓库
    std::string repo_path = ConfigManager::Instance().GetRepositoryPath();
    main_repo_ = git::OpenRepository(repo_path);
    if (!main_repo_) {
        wxMessageBox("Failed to open repository: " + repo_path, "Error", wxICON_ERROR);
        Close(true);
        return;
    }

    // 获取硬重置配置
    is_hard_reset_ = ConfigManager::Instance().GetHardReset();

    // 初始化UI
    InitializeUI();

    // 默认选择Develop分支组
    if (group_combo_->GetCount() > 0) {
        group_combo_->SetStringSelection("Develop");
        wxCommandEvent event(wxEVT_COMBOBOX);
        OnGroupSelect(event);
    }
}

void MainFrame::InitializeUI() {
    // 创建主布局
    wxBoxSizer* main_sizer = new wxBoxSizer(wxVERTICAL);

    // 创建分支组选择器
    PopulateBranchGroups();

    // 创建主仓库分支选择器
    branch_selectors_.push_back(new BranchSelector(this, main_repo_, "Main Repository"));

    // 创建子模块分支选择器
    auto submodules = main_repo_->GetSubmodules();
    for (const auto& submodule : submodules) {
        branch_selectors_.push_back(new BranchSelector(this, submodule));
    }

    // 创建硬重置选项
    hard_reset_checkbox_ = new wxCheckBox(this, wxID_ANY, "Hard reset");
    hard_reset_checkbox_->SetValue(is_hard_reset_);

    // 创建应用按钮
    wxButton* apply_button = new wxButton(this, wxID_APPLY, "Apply");

    // 创建日志控件
    log_ctrl_ = new ColoredLogCtrl(this);

    // 添加到主布局
    main_sizer->Add(group_combo_, 0, wxEXPAND | wxALL, 5);

    for (auto* selector : branch_selectors_) {
        main_sizer->Add(selector, 0, wxEXPAND | wxALL, 5);
    }

    main_sizer->Add(hard_reset_checkbox_, 0, wxALL, 5);
    main_sizer->Add(apply_button, 0, wxALL, 5);
    main_sizer->Add(log_ctrl_, 1, wxEXPAND | wxALL, 5);

    SetSizerAndFit(main_sizer);
    SetMinSize(wxSize(600, 400));
}

void MainFrame::PopulateBranchGroups() {
    // 获取所有分支组
    std::vector<std::string> branch_groups;

    // 获取主仓库的所有分支
    for (const auto& branch : main_repo_->GetAllBranches()) {
        branch_groups.push_back(branch);
    }

    // 获取子模块的所有分支
    auto submodules = main_repo_->GetSubmodules();
    for (const auto& submodule : submodules) {
        for (const auto& branch : submodule->GetAllBranches()) {
            branch_groups.push_back(branch);
        }
    }

    // 去重
    std::sort(branch_groups.begin(), branch_groups.end());
    branch_groups.erase(std::unique(branch_groups.begin(), branch_groups.end()), branch_groups.end());

    // 创建分支组选择器
    group_combo_ = new wxComboBox(this, wxID_ANY);

    // 填充分支组
    for (const auto& group : branch_groups) {
        group_combo_->Append(group);
    }
}

void MainFrame::OnApply(wxCommandEvent& event) {
    log_ctrl_->LogInfo("Applying branch changes...");

    bool any_failures = false;

    for (auto* selector : branch_selectors_) {
        std::string repo_name = selector->GetRepo()->GetName();
        std::string branch_name = selector->GetSelectedBranchName();

        log_ctrl_->LogInfo(wxString::Format("Switching %s to branch: %s", repo_name, branch_name));

        if (selector->ApplyBranchChange(is_hard_reset_)) {
            log_ctrl_->LogSuccess(wxString::Format("Successfully updated %s to %s", repo_name, branch_name));
        } else {
            log_ctrl_->LogError(wxString::Format("Failed to update %s to %s", repo_name, branch_name));
            any_failures = true;
        }
    }

    if (any_failures) {
        log_ctrl_->LogWarning("Some operations failed. See errors above.");
    } else {
        log_ctrl_->LogSuccess("All operations completed successfully!");
    }

    log_ctrl_->LogInfo("==============完成==============");
}

void MainFrame::OnGroupSelect(wxCommandEvent& event) {
    wxString selected_group = group_combo_->GetStringSelection();
    if (selected_group.IsEmpty()) return;

    log_ctrl_->LogInfo(wxString::Format("Selecting branch group: %s", selected_group));

    for (auto* selector : branch_selectors_) {
        selector->SelectGroupBranch(selected_group);
    }

    // 保存配置
    ConfigManager::Instance().Save();
}

void MainFrame::OnHardResetChanged(wxCommandEvent& event) {
    is_hard_reset_ = event.IsChecked();
    ConfigManager::Instance().SetHardReset(is_hard_reset_);

    if (is_hard_reset_) {
        log_ctrl_->LogWarning("Hard reset mode enabled. This will discard local changes!");
    } else {
        log_ctrl_->LogInfo("Hard reset mode disabled.");
    }
}
